//
//  NRViewController.m
//  ParseTwitterExample
//
//  Created by Nic Roth on 9/25/13.
//  Copyright (c) 2013 Nic Roth. All rights reserved.
//

#import "NRViewController.h"
#import <Parse/Parse.h>
#import <Accounts/Accounts.h>
#import <Social/Social.h>
#import "OAuth+Additions.h"
#import "TWAPIManager.h"
#import "TWSignedRequest.h"


@interface NRViewController ()

@end

@implementation NRViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    [self.twitterActivityIndicatorView stopAnimating];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - UX Actions

- (IBAction)tapTwitterLoginButton:(id)sender
{
    [self.twitterActivityIndicatorView startAnimating];
    self.twitterLoginButton.enabled = NO;
    self.twitterLoginButton.hidden = YES;
    
    self.outputLabel.text = @"Connecting...";
    
    [self authorizeUserWithTwitterAuthenticationAndCompletionBlock:^(BOOL completion, NSError *error) {
       
        //completion block
        
        if (!error)
        {
            //success!
            
            NSLog(@"Success.");
            
            [[[UIAlertView alloc] initWithTitle:@"Twitter Authenticated" message:@"You have authenticated with Twitter and created a PFUser." delegate:nil cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] show];
            
            
            NSString *output = [NSString stringWithFormat:@"Authenticated. \n Twitter Handle: %@", [[PFUser currentUser] objectForKey:@"TwitterHandle"]];
            
            self.outputLabel.text = output;

        }
        else
        {
            // there was an error
            
            NSLog(@"Error: %@", error.localizedDescription);
            self.twitterLoginButton.enabled = YES;
            self.twitterLoginButton.hidden = NO;
            
        }
        
        [self.twitterActivityIndicatorView stopAnimating];
    }];
}

#pragma mark - Twitter User Logic

- (void)authorizeUserWithTwitterAuthenticationAndCompletionBlock:(void (^)(BOOL completion, NSError *error))completion
{
    //sign in with twitter
    ACAccountStore *accountStore = [[ACAccountStore alloc] init];
    ACAccountType *accountType = [accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
    
    [accountStore requestAccessToAccountsWithType:accountType options:nil completion:^(BOOL granted, NSError *error) {
        if(granted) {
            NSArray *accountsArray = [accountStore accountsWithAccountType:accountType];
            
            if ([accountsArray count] > 0) {
                
                ACAccount *twitterAccount = [accountsArray objectAtIndex:0];
                [twitterAccount setAccountType:accountType];
                
                [TWAPIManager performReverseAuthForAccount:twitterAccount withHandler:^(NSData *responseData, NSError *error) {
                    if (responseData) {
                        NSString *responseStr = [[NSString alloc] initWithData:responseData encoding:NSUTF8StringEncoding];
                        
                        NSDictionary *dict = [self credentialsFromTwitterOAuthResponse:responseStr];
                        
                        
                        
                        NSString *oauth_token = [dict objectForKey:@"oauth_token"];
                        NSString *oauth_token_secret = [dict objectForKey:@"oauth_token_secret"];
                        NSString *screen_name = [dict objectForKey:@"screen_name"];
                        NSString *user_id = [dict objectForKey:@"user_id"];
                        
                        [PFTwitterUtils logInWithTwitterId:user_id screenName:screen_name authToken:oauth_token authTokenSecret:oauth_token_secret block:^(PFUser *user, NSError *error) {
                            if (!error) {
                                // success
                                NSLog(@"Success!");
                                
                                
                                
                                //grab the data
                                NSURL *reqURL = [NSURL URLWithString:@"https://api.twitter.com/1.1/users/show.json"];
                                NSDictionary *params = @{@"screen_name": screen_name, @"user_id": user_id};
                                
                                SLRequest *request = [SLRequest requestForServiceType:SLServiceTypeTwitter requestMethod:SLRequestMethodGET URL:reqURL parameters:params];
                                
                                [request setAccount:twitterAccount];
                                
                                [request performRequestWithHandler:^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error) {
                                    
                                    if (!error) {
                                        NSDictionary *info = [NSJSONSerialization JSONObjectWithData:responseData options:kNilOptions error:&error];
                                        NSLog(@"Info: %@", info);
                                        
                                        
                                        
                                        //parse it
                                        NSString *name = [info objectForKey:@"name"];
                                        NSString *profilePic = [info objectForKey:@"profile_image_url"];
                                        
                                        
                                        [user setObject:name forKey:@"Name"];
                                        [user setObject:profilePic forKey:@"TwitterImageURL"];
                                        [user setObject:screen_name forKey:@"TwitterHandle"];
                                        [user save];
                                        
                                        
                                        dispatch_async(dispatch_get_main_queue(), ^{
                                            NSLog(@"Done!");
                                            
                                            
                                            // fire completion block
                                            completion(YES, nil);
                                            
                                            //[PFUser logOut];
                                        });
                                        
                                        
                                    } else {
                                        NSLog(@"ERR: %@", error);
                                        
                                        // fire completion block
                                        completion(NO, error);
                                        
                                        
                                    }
                                }];
                                
                                
                                
                            } else {
                                NSLog(@"ERR: %@", error);
                                
                                // fire completion block
                                completion(NO, error);
                                
                            }
                        }];
                        
                    }
                    else {
                        NSLog(@"Error!\n%@", [error localizedDescription]);
                        
                        // fire completion block
                        completion(NO, error);
                        
                    }
                }];
                
                
            }
        }
    }];
}

- (NSDictionary *)credentialsFromTwitterOAuthResponse:(NSString *)response
{
    
    // Divide the string by ampersands
    NSArray *components = [response componentsSeparatedByString:@"&"];
    
    NSMutableDictionary *credentials = [NSMutableDictionary dictionaryWithCapacity:4];
    
    // Iterate through the components
    for (NSString *component in components) {
        
        // Split by = sign
        NSRange divider = [component rangeOfString:@"="];
        NSString *key = [component substringToIndex:divider.location];
        NSString *val = [component substringFromIndex:divider.location+1];
        [credentials setObject:val forKey:key];
    }
    
    return credentials;
}

@end
