ParseTwitterExample
===================

An example project demonstrating native iOS authentication for Twitter that creates a basic PFUser object on a Parse.com backend.


Inspired by an [answer](https://www.parse.com/questions/ios-builtin-twitter-integration "Parse Help") by Héctor Ramos on Parse's help forum. 


### Credits

TWReverseAuth - [GitHub](https://github.com/seivan/TWReverseAuth)  
ABOAuthCore - [Bitbucket](https://bitbucket.org/atebits/oauthcore)  
LUKeychainAccess - [GitHub](https://github.com/TheLevelUp/LUKeychainAccess)  
NSData+Base64 - [Matt Gallagher](http://www.cocoawithlove.com/2009/06/base64-encoding-options-on-mac-and.html)  
TWReverseAuthExample - [GitHub](https://github.com/seancook/TWReverseAuthExample)  

### How to do it yourself

##### Project Setup

Follow Parse's [Quickstart Guide](https://parse.com/apps/quickstart#ios/native/existing) for iOS projects.  

* Include `Parse.framework`  
* Add the required dependencies 

Add the following for Twitter. If you already have a Parse project, you can probably start here. 

* `Accounts.framework`
* `Social.framework`
* `Twitter.framework`

Include the necessary dependencies referenced above in the Credits section.

* `ABOAuthCore` be sure to add `-fno-objc-arc` as a build flag
* `TWReverseAuth`
* `LUKeychainAccess`
* `NSData+Base64` be sure to add `-fno-objc-arc` as a build flag


##### Configure Twitter

Set your Twitter App's credentials in `TWAppCredentialStore`. 

In your AppDelegate file, you'll need to configure `PFTwitterUtils` and `TWAPIManager`. 

```objective-c
// Parse Twitter setup
[PFTwitterUtils initializeWithConsumerKey:kTWAppKey consumerSecret:kTWAppSecret];
[TWAPIManager registerTwitterAppKey:kTWAppKey andAppSecret:kTWAppSecret];
```

##### Twitter authentication logic

In the example project, I've created a very useful block-based method for (1) authenticating via Twitter, (2) creating a `PFUser`, and (3) adding some basic information about the Twitter account to the `PFUser`. This is wrapped in 
 
```objective-c
- (void)authorizeUserWithTwitterAuthenticationAndCompletionBlock:(void (^)(BOOL completion, NSError *error))completion;
```

You can probably just copy this method, adjust it as necessary, and then call it in your UI logic. In the example project, I show how to call the method and handle some basic UI logic. Here is my UI-related logic, for reference. It fires a simple `UIAlertView` on completion, or logs an error.

```objective-c
- (IBAction)tapTwitterLoginButton:(id)sender
{
    [self.twitterActivityIndicatorView startAnimating];
    self.twitterLoginButton.enabled = NO;
    self.twitterLoginButton.hidden = YES;
    
    self.outputLabel.text = @"Connecting...";
    
    [self authorizeUserWithTwitterAuthenticationAndCompletionBlock:^(BOOL completion, NSError *error) {
       
        //completion block
        
        if (!error)
        {
            //success!
            
            NSLog(@"Success.");
            
            [[[UIAlertView alloc] initWithTitle:@"Twitter Authenticated" message:@"You have authenticated with Twitter and created a PFUser." delegate:nil cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] show];
            
            
            NSString *output = [NSString stringWithFormat:@"Authenticated. \n Twitter Handle: %@", [[PFUser currentUser] objectForKey:@"TwitterHandle"]];
            
            self.outputLabel.text = output;

        }
        else
        {
            // there was an error
            
            NSLog(@"Error: %@", error.localizedDescription);
            self.twitterLoginButton.enabled = YES;
            self.twitterLoginButton.hidden = NO;
            
        }
        
        [self.twitterActivityIndicatorView stopAnimating];
    }];
}
```

### Need help?
<n@nr.cm> or tweet me [@nicroth](http://twitter.com/nicroth)
